/*******************************************************************************
 * 
 * Cria processos e muda suas prioridades.
 *
 ******************************************************************************/

#include"RT.h"

pid_t createProcess(char *command, char *params[])
{
    pid_t pid;

    pid = fork();
    if (pid < 0) {
        perror("fork");
        exit(1);
    }
    if (pid == 0) {
        setupClientFIFOs();
        changeClientPrio(getpid());
        if (setProcessAffinity(pid) == -1) {
            printf("\nNão foi possível configurar afinidade de processador. Continuando...");
        }
        execvp(command, params);
        exit(0);
    }
    kill(pid, SIGSTOP);
    return pid;
}

void printSchedInfo()
{
    int sched, nice;
    struct sched_param param;

    printf("Scheduler for pid %d (server): ", getpid());
    sched = sched_getscheduler(0);
    switch (sched) {
    case SCHED_RR:
        sched_getparam(0, &param);
        printf("SCHED_RR, sched_priority = %d\n", param.sched_priority);
        break;
    case SCHED_OTHER:
        printf("SCHED_OTHER, ");
        errno = 0;
        nice = getpriority(PRIO_PROCESS, 0);
        if (nice == -1 && errno != 0)
            printf("getpriority error.\n");
        else
            printf("nice = %d\n", nice);
        break;
    }
}

void setMaxOther()
{
    int minNice, maxNice, i;
    int res = -1;
    struct sched_param params;

    minNice = -NZERO;
    maxNice = getpriority(PRIO_PROCESS, 0);

    for (i = minNice; i < maxNice; i++) {
        params.sched_priority = i;
        res = setpriority(PRIO_PROCESS, 0, i);
        if (res != -1)
            return;
    }
}

int setMaxRT()
{
    int maxPrio, minPrio, i;
    int res = -1;
    struct sched_param params;

    maxPrio = sched_get_priority_max(SCHED_RR);
    minPrio = sched_get_priority_min(SCHED_RR);

    for (i = maxPrio; i >= minPrio; i--) {
        params.sched_priority = i;
        res = sched_setscheduler(getpid(), SCHED_RR, &params);
        if (res != -1)
            return 1;
    }
    return 0;
}

void changeServerPrio()
{
    if (!setMaxRT())
        setMaxOther();
    printSchedInfo();
}

void changeClientPrio(pid_t pid)
{
    struct sched_param param;

    param.sched_priority = 0;
    sched_setscheduler(pid, SCHED_OTHER, &param);
    changeNice(NZERO - 1, pid);
}

void changeNice(int value, pid_t pid)
{
    int res;

    /* Nice will have "value" value */
    res = setpriority(PRIO_PROCESS, pid, value);
    if (res == -1) {
        printf("Não foi possível: ");
        if (errno == EPERM)
            printf("usuário deve ser administrador.");
        else if (errno == EINVAL)
            printf("parâmetros inválidos.");
        else if (errno == ESRCH)
            printf("processo não encontrado.");
        printf("Abortando.\n");
        exit(1);
    }
}
