/*******************************************************************************
 * 
 * Arquivo contendo métodos para a manipulação de listas ligadas. Existem duas 
 * listas, uma contendo processos sendo executados e outra contendo o próximo 
 * deadline de execução para cada processo.
 *
 ******************************************************************************/
#ifndef LISTS_H
#define LISTS_H

#include<stdio.h>
#include<unistd.h>
#include<sys/time.h>
#include<time.h>
#include<stdlib.h>
#include<semaphore.h>
#include<signal.h>

struct processDataStruct {
    pid_t pid;
    unsigned long slice;
    unsigned long next_period;
    unsigned long executed_time;
    int running;
    int num_child;
    sem_t childListSem;
    struct pidDataStruct *childrenList;
    struct processDataStruct *next;
};

struct pidDataStruct {
    pid_t pid;
    struct pidDataStruct *next;
};

struct alarmDataStruct {
    unsigned long alarmTime;
    struct processDataStruct *process;
    struct alarmDataStruct *next;
};

#include "macros.h"
#include "reserve.h"
#include "calcSlices.h"

int checkProcessExistance(pid_t pid);
int createListsSemaphores();
void insertAtAlarmList(struct processDataStruct *process, unsigned long time);
void killProcesses();
void killProcessTree(pid_t pid);
void killProcessTreeRec(struct pidDataStruct *node);
struct processDataStruct *addClientToList(pid_t pid);
void removeClientFromList(pid_t pid);
void removeFromAlarmList(struct processDataStruct *process);
int removeFromAlarmListHead(struct alarmDataStruct *alarm);
struct processDataStruct *getProcessFromPid(pid_t pid);

#endif
