/*******************************************************************************
 * 
 * Arquivo contendo métodos que implementam a manipulação dos processos ativos
 * para que os mesmos sejam executados nos períodos de tempo pré-determinados.
 *
 ******************************************************************************/
#ifndef RESERV_H
#define RESERV_H

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <sched.h>
#include <stdio.h>
#include <sys/time.h>
#include <signal.h>
#include <pthread.h>

#include "monitor.h"
#include "socket.h"
#include "alarms.h"
#include "calcSlices.h"
#include "lists.h"

#define RESERVATION_LIMIT 1.0
#define NUMBER_OF_PROCESSORS sysconf(_SC_NPROCESSORS_CONF)

timer_t created_timer;
sem_t reservationListSem;

struct processDataStruct;

struct reservationDataStruct {
    unsigned long period;
    unsigned long slice;
    pid_t pid;
    struct reservationDataStruct *next;
};

int createTimer();
void restartProcessMonitor(struct processDataStruct *process);
int setProcessAffinity(pid_t pid);
int setTimer(long interval_sec, long interval_usec, long value_sec, long value_usec);
void startMonitorClient(struct processDataStruct *process);
void stopMonitorClient(struct processDataStruct *process);
void stopProcessTree(struct processDataStruct *process);
void treatReserveMessage(struct reserveMsgDataStruct *msgData);
void updateProcessesAffinity();
void updateSlices(struct processDataStruct *processList, sliceListT sliceList);
void wakeUpProcessTree(struct processDataStruct *process);

#endif
