/*******************************************************************************
 * 
 * Arquivo com métodos para a manipulação de envio/recebimento de mensagens via 
 * sockets.
 *
 ******************************************************************************/

#include"socket.h"

sem_t socketSem;

/** Método para a criação de um socket por onde os clientes se conectarão ao 
servidor */
int createSvrSocket(int portNumber)
{
    struct sockaddr_in serv_addr;
    int sockfd;

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERRO ao abrir o socket");

    /* Inicializa serv_addr com zero */
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(portNumber);

    if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0)
        error("ERRO no binding");
    listen(sockfd, 20);

    return sockfd;
}

/** Método para a criação dos sockets dos clientes */
int createCliSocket(char *host, int portNumber)
{
    int sockfd;
    struct sockaddr_in serv_addr;
    struct hostent *server;

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERRO ao abrir o socket");

    server = gethostbyname(host);

    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *) server->h_addr, (char *) &serv_addr.sin_addr.s_addr, server->h_length);
    serv_addr.sin_port = htons(portNumber);

    if (connect(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr))
        < 0)
        error("ERRO ao conectar");

    return sockfd;
}

/** Método que cria uma conexão cliente-servidor */
int acceptConnection(int sockfd)
{
    socklen_t clilen;
    int newsockfd;
    struct sockaddr_in cli_addr;

    sem_wait(&socketSem);
    clilen = sizeof(cli_addr);
    do {
        newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
    } while (newsockfd < 0 && errno == EINTR);
    if (newsockfd < 0)
        printf("ERRO no accept.\n");
    sem_post(&socketSem);

    return newsockfd;
}

/** Método pelo qual o servidor recebe a opção de reservar ou adaptar processo 
    do cliente */
void recvClientOption(int newsockfd, char *msg)
{
    int n;

    n = read(newsockfd, msg, 255);
    if (n < 0)
        printf("ERRO na leitura do socket");

    printf("Opção recebida: %s\n", msg);
}

/* O servidor recebe do cliente o PID cuja árvore deve ser finalizada */
pid_t recvKillMsg(int newsockfd)
{
    char buffer[32];
    char *token;
    int n;
    pid_t pid;

    n = read(newsockfd, buffer, 32);
    if (n < 0)
        printf("ERRO na leitura do socket");

    token = strtok(buffer, " ");
    pid = atoi(token);

    return pid;
}

/** Método que recebe uma mensagem de reserva do cliente e a processa */
void recvReserveMsg(int newsockfd, struct reserveMsgDataStruct *msgData)
{
    char buffer[256];
    char *tempBuffer = NULL;
    char *token;
    int n;
    int i;

    buffer[0] = '\0';
    n = read(newsockfd, buffer, 255);
    if (n < 0)
        printf("ERRO na leitura do socket");

    printf("\nMensagem recebida: %s\n", buffer);

    msgData->command[0] = '\0';

    token = strtok_r(buffer, " ", &tempBuffer);
    strcpy(msgData->command, token);
    token = strtok_r(NULL, " ", &tempBuffer);
    if (token == NULL)
        msgData->params[0] = NULL;
    else {
        i = 0;
        msgData->params[i] = (char *) malloc(16 * sizeof(char));
        strcpy(msgData->params[i++], msgData->command);
        msgData->params[i] = (char *) malloc(16 * sizeof(char));
        strcpy(msgData->params[i++], token);
        token = strtok_r(NULL, " ", &tempBuffer);
        while (token != NULL) {
            msgData->params[i] = (char *) malloc(16 * sizeof(char));
            strcpy(msgData->params[i++], token);
            token = strtok_r(NULL, " ", &tempBuffer);
        }
        msgData->params[i] = NULL;
    }
}

/** Método que recebe uma mensagem para o servidor e a processa */
void recvServerAdaptMsg(int newsockfd, struct serverAdaptMsgDataStruct *msgData)
{
    char buffer[256];
    char *tempBuffer = NULL;
    char *token;
    int n;

    buffer[0] = '\0';
    n = read(newsockfd, buffer, 255);
    if (n < 0)
        printf("ERRO na leitura do socket");

    printf("\nMensagem recebida: %s\n", buffer);

    token = strtok_r(buffer, " ", &tempBuffer);
    msgData->mask = atoi(token);
    token = strtok_r(NULL, " ", &tempBuffer);
    msgData->processingLimit = atoi(token);
}

/** Método pelo qual servidor envia mensagem ao cliente*/
void sndMsgClient(int sockfd, char *msg, int msgLen)
{
    int n;

    n = write(sockfd, msg, msgLen);
    if (n < 0)
        printf("ERRO ao escrever no socket");
}

/** Método pelo qual o cliente envia opção de reservar ou adaptar processo ao 
    servidor */
void sendClientOption(int sockfd, char *option)
{
    int n;
    char buffer[256];

    buffer[0] = '\0';
    n = write(sockfd, option, strlen(option) + 1);
    if (n < 0)
        error("ERRO ao escrever no socket");
    n = read(sockfd, buffer, 255);
    if (n < 0)
        error("ERRO na leitura do socket");
    printf("%s", buffer);
}

/* O cliente envia o PID do processo que deve ser finalizado pelo servidor */
void sendKillMsg(int sockfd, pid_t pid)
{
    char buffer[32];
    int n;

    sprintf(buffer, "%d", pid);
    n = write(sockfd, buffer, strlen(buffer) + 1);
    if (n < 0)
        error("ERRO na leitura do socket");
}

void sendACK(int sockfd)
{
    char dummy[1];
    int n;

    dummy[0] = '\0';
    n = write(sockfd, dummy, 1);
    if (n < 0)
        printf("ERRO ao escrever no socket");
}

void receiveACK(int sockfd)
{
    char dummy[1];
    int n;

    n = read(sockfd, dummy, 1);
    if (n < 0)
        printf("ERRO na leitura do socket");
}

/** Método pelo qual o cliente envia mensagem de reserva ao servidor */
int sendReserveMsg(int sockfd, char *command, char *params)
{
    char buffer[256];
    char buffer2[256];
    int n;
    int pid;

    pid = -1;
    buffer[0] = '\0';

    strcat(buffer, command);
    strcat(buffer, " ");
    strcat(buffer, params);

    n = write(sockfd, buffer, strlen(buffer) + 1);
    if (n < 0)
        error("ERRO ao escrever no socket");

    buffer2[0] = '\0';
    n = read(sockfd, buffer2, 255);
    if (n < 0)
        error("ERRO na leitura do socket");

    sscanf(buffer2, "%d", &pid);
    return pid;
}

/** Método pelo qual o usuário altera opções do servidor. */
void sendServerAdaptMsg(int sockfd, char *strMask, char *processingLimit)
{
    char buffer[256];
    char buffer2[256];
    int n;

    buffer[0] = '\0';
    strcat(buffer, strMask);
    strcat(buffer, " ");
    strcat(buffer, processingLimit);

    n = write(sockfd, buffer, strlen(buffer) + 1);
    if (n < 0)
        error("ERRO ao escrever no socket");

    buffer2[0] = '\0';
    n = read(sockfd, buffer2, 255);
    if (n < 0)
        error("ERRO na leitura do socket");
    printf("%s\n", buffer2);
}

void closeSocket(int sockfd)
{
    if (close(sockfd) == -1)
        printf("ERRO ao fechar o socket");
}

int createSocketSemaphore()
{
    if (sem_init(&socketSem, 0, 1) == -1)
        return -1;
    return 1;
}
