////////////////////////////////////////////////////////////////////////////////////////////
// Project: NCC InteGrade Interface                           //
// Developed by: Giulian Dalton Luz & Rogerio Guaraci dos Santos              //
// Date: 8/12/2003                                    //
// Version: Beta 1                                    //
// Module Description: Socket.cpp. Implements all socket features.                //
////////////////////////////////////////////////////////////////////////////////////////////

#include "Socket.h"
#include "string.h"
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <iostream>

Socket::Socket()
{
	memset ( &m_addr,
		0,
		sizeof ( m_addr ) );
}


void Socket::close()
{
	closeListening();
	closeConnection();
}


void Socket::closeListening()
{
	::close ( listenFd );
}


void Socket::closeConnection()
{
	shutdown(connFd, SHUT_WR);
	::close ( connFd );
}


bool Socket::create()
{
	listenFd = socket ( AF_INET,
		SOCK_STREAM,
		0 );

	if ( ! is_valid() )
		return false;

	// TIME_WAIT - argh
	int on = 1;
	if ( setsockopt ( listenFd, SOL_SOCKET, SO_REUSEADDR, ( const char* ) &on, sizeof ( on ) ) == -1 )
		return false;

	return true;
}


bool Socket::bind ( const int port )
{

	if ( ! is_valid() ) {
		return false;
	}

	m_addr.sin_family = AF_INET;
	m_addr.sin_addr.s_addr = INADDR_ANY;
	m_addr.sin_port = htons ( port );

	int bind_return = ::bind ( listenFd,
		( struct sockaddr * ) &m_addr,
		sizeof ( m_addr ) );

	if ( bind_return == -1 ) {
		return false;
	}

	return true;
}


bool Socket::listen() const
{
	if ( ! is_valid() ) {
		return false;
	}

	int listen_return = ::listen ( listenFd, MAXCONNECTIONS );

	if ( listen_return == -1 ) {
		return false;
	}

	return true;
}


bool Socket::accept()
{
	int addr_length = sizeof ( m_addr );
	connFd = ::accept ( listenFd, ( sockaddr * ) &m_addr, ( socklen_t * ) &addr_length );

	if ( connFd <= 0 )
		return false;
	else
		return true;
}


bool Socket::send ( const std::string s ) const
{
	int status = ::send ( connFd, s.c_str(), s.size(), MSG_NOSIGNAL );
	if ( status == -1 ) {
		return false;
	}
	else {
		return true;
	}
}


int Socket::recv ( std::string& s ) const
{
	char buf [ MAXRECV + 1 ];

	s = "";

	memset ( buf, 0, MAXRECV + 1 );

	int status = ::recv ( connFd, buf, MAXRECV, 0 );

	if ( status == -1 ) {
		std::cout << "status == -1   errno == " << errno << "  in Socket::recv\n";
		return 0;
	}
	else if ( status == 0 ) {
		return 0;
	}
	else {
		s = buf;
		return status;
	}
}


bool Socket::connect ( const std::string host, const int port )
{
	if ( ! is_valid() ) return false;

	m_addr.sin_family = AF_INET;
	m_addr.sin_port = htons ( port );

	int status = inet_pton ( AF_INET, host.c_str(), &m_addr.sin_addr );

	if ( errno == EAFNOSUPPORT ) return false;

	status = ::connect ( listenFd, ( sockaddr * ) &m_addr, sizeof ( m_addr ) );

	if ( status == 0 )
		return true;
	else
		return false;
}


void Socket::set_non_blocking ( const bool b )
{

	int opts;

	opts = fcntl ( listenFd,
		F_GETFL );

	if ( opts < 0 ) {
		return;
	}

	if ( b )
		opts = ( opts | O_NONBLOCK );
	else
		opts = ( opts & ~O_NONBLOCK );

	fcntl ( listenFd,
		F_SETFL,opts );

}
