////////////////////////////////////////////////////////////////////////////////////////////
// Project: NCC InteGrade Interface                           //
// Developed by: Giulian Dalton Luz & Rogerio Guaraci dos Santos              //
// Date: 8/12/2003                                    //
// Version: Beta 1                                    //
// Module Description: ncc.cpp. Main Module. Contains ncc integration and webserver       //
// treatment. Needs refactoring for next version.                     //
////////////////////////////////////////////////////////////////////////////////////////////

#include "ServerSocket.h"
#include "SocketException.h"
#include <string>
#include <string.h>
#include <sstream>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <map>
#include "CPU.h"
#include "Log.h"
#include "ProcessList.h"
#include "ProcessListPage.h"
#include "TrafficLightPage.h"

using namespace std;

//Vars to simulate InteGrade
int iLimitMemory = 128;
int iSemaphState = 1;
int iLimitDisk = 1024;
int iLimitCPU = 50;
string sched[4][100];
int iSched = 0;

//functions body
void getFile(const char *, ServerSocket);
std::string getNCC(const char *, string&);
std::string readFile(const char *);
string& replaceAll(string&, const string&, const string&);
std::string getProcessList();
std::string getScheduleList();
std::string getDay(string);
std::string getAvail(string);
int validHour(string);

// main
int main ( int argc, char* argv[] )
{
	std::string data;

	Log log("ncc.cpp");
	log.clear();

	//simulate two schedules stored in InteGrade
	sched[0][iSched] = "10:00";
	sched[1][iSched] = "23:59";
	sched[2][iSched] = "1";
	sched[3][iSched] = "1";
	iSched++;
	sched[0][iSched] = "08:00";
	sched[1][iSched] = "10:00";
	sched[2][iSched] = "0";
	sched[3][iSched] = "0";
	iSched++;

	//the lines below contais the web server inicialization (opens a socket server)
	std::cout << "running on port " << argv[1]<< "....\n";

	try
	{
		// Create the socket
		ServerSocket server ( atoi(argv[1]) );
		while ( true ) {
			try
			{
				while ( true ) {
					server.accept();
					data.clear();
					server >> data;

					std::cout << data << "\n";
					std::string sGet(data.substr(4,data.find(" ", 4)-4));
					std::string sFile;
					std::string sData;
					if (sGet.find("?") != string::npos) {
						sFile = sGet.substr(0,sGet.find("?"));
						sData = sGet.substr(sGet.find("?")+1);
					}
					else
						sFile = sGet;

					if (sFile == "/")
						sFile.append("index.ncc");

					sFile = "./web" + sFile;

					if (sFile.substr(sFile.size()-3,3) == "ncc") {
						server << getNCC(sFile.c_str(), sData);
					}
					else {
						getFile(sFile.c_str(), server);
					}
					server.closeSocket();
				}
			}
			catch ( SocketException& ) {}
		}
		server.close();
	}
	catch ( SocketException& e ) {
		std::cout << "Exception was caught:" << e.description() << "\nExiting.\n";
	}

	return 0;
}


//Reads contentes of an file and write in socket
void getFile(const char * sFileName, ServerSocket ss)
{
	string s;
	s = readFile(sFileName);

	char buffer[255];
	getcwd(buffer, 255);
	string buf = buffer;
	string sNewPath = "src=\"file:" + buf + "/web/";
	s = replaceAll(s, "src=\"", sNewPath);
	ss << s;
}


//Reads contentes of an .ncc file
//makes all ncc services
//and write in socket
std::string getNCC(const char * sFileName, string& sData)
{
	string s;
	string tag;
	string value;
	string sError;
	mapStSt queryString;
	mapStSt vars;
	int i;
	int j=0;

	//reads ncc file
	s = readFile(sFileName);

	/* New approach to deliver web pages.
	 * For each page, create the necessary objects.
	 * The objects are responsable to parse NCC template string.
	 * They receive query string as a map */
	if (strcmp(sFileName, "./web/resourcemanager.ncc") == 0) {
		CPU cpu(s);
		cpu.setQueryString(sData);
		s = cpu.getPage();
	}
	else if (strcmp(sFileName, "./web/process.ncc") == 0) {
		CPUReserve cpur(PORT);

		cpur.updateStatus();
		ProcessList *procList = cpur.getProcessList();
		procList->parseTopOutput(cpur.getNumberOfCPUs());

		string processTpl = readFile("./web/aprocess.inc");
		string processesTpl = readFile("./web/process.ncc");
		ProcessListPage procListPage(processesTpl, processTpl, procList);
		procListPage.setQueryString(sData);
		s = procListPage.getPage();
	}

	/*
	 * BEGIN old approach (compatibility reasons)
	 */
	i = sData.find("=", j);
	//reads parameters of GET method
	while (i != (int) string::npos) {
		tag = sData.substr(j, i-j);
		j = i + 1;
		i = sData.find("&", i);
		if (i != (int) string::npos ) {
			value = sData.substr(j, i-j);
			j = i + 1;
		}
		else {
			value = sData.substr(j);
			j = sData.size();
		}
		i = sData.find("=", j);
		if (tag == "txtMemory")
			iLimitMemory = atoi(value.c_str());
		else if (tag == "txtDisk")
			iLimitDisk = atoi(value.c_str());
		else if (tag == "txtCPU")
			iLimitCPU = atoi(value.c_str());
		else if (tag == "newsemaphstate")
			iSemaphState = atoi(value.c_str());
		else if (tag == "txtStart") {
			value = replaceAll(value, "%3A", ":");
			if (validHour(value) == 1) {
				sched[0][iSched] = value;
			}
			else {
				sError = "Invalid start hour. try hh:mm!";
				break;
			}
		}
		else if (tag == "txtEnd") {
			value = replaceAll(value, "%3A", ":");
			if (validHour(value) == 1) {
				sched[1][iSched] = value;
			}
			else {
				sError = "Invalid end hour. try hh:mm!";
				break;
			}
		}
		else if (tag == "selWeekDay")
			sched[2][iSched] = replaceAll(value, "%3A", ":");
		else if (tag == "selState")
			sched[3][iSched] = replaceAll(value, "%3A", ":");
		else if (tag == "newSched") {
			sError = "Schedule added!!!";
			iSched++;
		}
		else if (tag == "Remove") {
			int k;
			for (k=atoi(value.c_str()); k < iSched-1; k++) {
				sched[0][k] = sched[0][k+1];
				sched[1][k] = sched[1][k+1];
				sched[2][k] = sched[2][k+1];
				sched[3][k] = sched[3][k+1];
			}
			iSched--;
		}
	}

	TrafficLightPage trafficLight(readFile("./web/sem.inc"));

	//make includes
	s = replaceAll(s, "<@TOOLBAR>", readFile("./web/toolbar.inc"));
	s = replaceAll(s, "<@SEMAPHORE>", trafficLight.getPage());

	//gets states of resource permission (traffic light)
	std::ostringstream semaph;
	semaph << iSemaphState;
	s = replaceAll(s, "<@SEMAPHSTATE>", semaph.str());

	//reads limits stored in InteGrade (simulated)
	std::ostringstream x1;
	x1 << iLimitMemory;
	s = replaceAll(s, "<@LIMITMEMORY>", x1.str());
	std::ostringstream x2;
	x2 << iLimitDisk;
	s = replaceAll(s, "<@LIMITDISK>", x2.str());
	std::ostringstream x3;
	x3 << iLimitCPU;
	s = replaceAll(s, "<@LIMITCPU>", x3.str());

	//reads available resources given by InteGrade (simulated)
	s = replaceAll(s, "<@AVAILABLEMEMORY>", "512");
	s = replaceAll(s, "<@AVAILABLEDISK>", "7262");
	s = replaceAll(s, "<@AVAILABLECPU>", "100");

	//gets a list of schedules
	s = replaceAll(s, "<@SCHEDULE>", getScheduleList());

	//shows an message in place of <@ERROR> tag
	s = replaceAll(s, "<@ERROR>", sError);

	//replaces scr of html tags to current path to allows more performance
	//Considering that this is a local application
	char buffer[255];
	getcwd(buffer, 255);
	string buf = buffer;
	/* 
	 * END old approach
	 */

	return(s);
}


//reads the contens of an file
std::string readFile(const char * sFileName)
{
	std::string ret;
	ifstream in(sFileName);
	std::string s;
	while(getline(in,s)) {
		ret.append(s);
		ret.append("\n");
	}
	return(ret);
}


//replace all ocurrences of "from"  in "contents" using "to"
string& replaceAll(string& context, const string& from, const string& to)
{
	size_t lookHere = 0;
	size_t foundHere;

	while ((foundHere = context.find(from, lookHere)) != string::npos) {
		context.replace(foundHere, from.size(), to);
		lookHere = foundHere + to.size();
	}
	return context;
}


//Simulate integration with InteGrade to get a list of schedules
std::string getScheduleList()
{
	int i;
	std::string ret;
	std::string aux;
	std::string temp;
	aux = readFile("./web/aschedule.inc");

	for (i=0; i < iSched; i++) {
		temp = aux;
		temp = replaceAll(temp, "<@START>", sched[0][i]);
		temp = replaceAll(temp, "<@END>", sched[1][i]);
		temp = replaceAll(temp, "<@DAY>", getDay(sched[2][i]));
		temp = replaceAll(temp, "<@STATE>", getAvail(sched[3][i]));
		std::ostringstream x;
		x << i;
		temp = replaceAll(temp, "<@NUMBER>", x.str());
		ret.append(temp);
	}

	return ret;
}


//Auxiliar function to convert an number to an week day description
std::string getDay(string s)
{
	if (s == "0") return "All Days";
	if (s == "1") return "Monday";
	if (s == "2") return "Tuesday";
	if (s == "3") return "Wednesday";
	if (s == "4") return "Thursday";
	if (s == "5") return "Friday";
	if (s == "6") return "Saturday";
	return "Sunday";
}


//Auxiliar function to convert an number to available or unavailable
std::string getAvail(string s)
{
	if (s == "1") return "Available";
	return "Unavailable";
}


//valids an string that may contains a time in hh:mm format
int validHour(string h)
{
	int ret = 0;
	std::ostringstream x1;
	x1 << atoi(h.substr(0,1).c_str());
	if (! (x1.str() == h.substr(0,1)))
		return 0;

	std::ostringstream x2;
	x2 << atoi(h.substr(1,1).c_str());
	if (! (x2.str() == h.substr(1,1)))
		return 0;

	std::ostringstream x3;
	x3 << atoi(h.substr(3,1).c_str());
	if (! (x3.str() == h.substr(3,1)))
		return 0;

	std::ostringstream x4;
	x4 << atoi(h.substr(4,1).c_str());
	if (! (x4.str() == h.substr(4,1)))
		return 0;

	if (atoi(h.substr(0,2).c_str()) < 24 && atoi(h.substr(0,2).c_str()) >= 0) {
		if (h.substr(2,1) == ":") {
			if (atoi(h.substr(3,2).c_str()) < 60 && atoi(h.substr(3,2).c_str()) >= 0) {
				ret = 1;
			}
		}
	}

	return ret;
}
