#include <iostream>
#include <cppunit/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/extensions/TestFactoryRegistry.h>

#include "../Log.h"
#include "../CPUReserve.h"

class CPUReserveTest : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE(CPUReserveTest);
	/* Test Cases */
	CPPUNIT_TEST(shouldWarnIfThereIsNoServer);
	CPPUNIT_TEST(shouldParseStatus);
	CPPUNIT_TEST_SUITE_END();

	protected:
		void shouldWarnIfThereIsNoServer() {
			CPUReserve cpur(1);
			CPPUNIT_ASSERT_EQUAL(false, cpur.updateStatus());
		}

		void shouldParseStatus() {
			string statusOut ("CPU: 3 2 67\nProcesses:\n");
			statusOut += "-123\n--456\n";
			statusOut += "-111\n--222\n--333";

			CPUReserve cpur(1);
			cpur.parseStatus(statusOut);
			CPPUNIT_ASSERT_EQUAL(3, cpur.getMask()  );
			CPPUNIT_ASSERT_EQUAL(2, cpur.getNumberOfCPUs()  );
			CPPUNIT_ASSERT_EQUAL(67, cpur.getCPULimit()  );

			ProcessList *procList = cpur.getProcessList();
            Process *proc = procList->getProcessByPid(123);
			CPPUNIT_ASSERT_EQUAL(456, proc->getChildren()->at(0)->getPid());
            proc = procList->getProcessByPid(111);
			CPPUNIT_ASSERT_EQUAL(222, proc->getChildren()->at(0)->getPid());
			CPPUNIT_ASSERT_EQUAL(333, proc->getChildren()->at(1)->getPid());
		}
};

CPPUNIT_TEST_SUITE_REGISTRATION(CPUReserveTest);

int main( int ac, char **av )
{
	//--- Create the event manager and test controller
	CPPUNIT_NS::TestResult controller;

	//--- Add a listener that colllects test result
	CPPUNIT_NS::TestResultCollector result;
	controller.addListener( &result );

	//--- Add a listener that print dots as test run.
	CPPUNIT_NS::BriefTestProgressListener progress;
	controller.addListener( &progress );

	//--- Add the top suite to the test runner
	CPPUNIT_NS::TestRunner runner;
	runner.addTest( CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest() );
	runner.run( controller );

	return result.wasSuccessful() ? 0 : 1;
}
